@@ -149,11 +149,15 @@ static void do_verify_push_status(record_callbacks_data *data, const push_status
 
 		cl_fail(git_buf_cstr(&msg));
 
-		git_buf_free(&msg);
+		git_buf_dispose(&msg);
 	}
 
-	git_vector_foreach(actual, i, iter)
-		git__free(iter);
+	git_vector_foreach(actual, i, iter) {
+		push_status *s = (push_status *)iter;
+		git__free(s->ref);
+		git__free(s->msg);
+		git__free(s);
+	}
 
 	git_vector_free(actual);
 }
@@ -263,8 +267,8 @@ failed:
 		git__free(actual_ref);
 
 	git_vector_free(&actual_refs);
-	git_buf_free(&msg);
-	git_buf_free(&ref_name);
+	git_buf_dispose(&msg);
+	git_buf_dispose(&ref_name);
 }
 
 static void verify_update_tips_callback(git_remote *remote, expected_ref expected_refs[], size_t expected_refs_len)
@@ -309,8 +313,8 @@ failed:
 	if (failed)
 		cl_fail(git_buf_cstr(&msg));
 
-	git_buf_free(&ref_name);
-	git_buf_free(&msg);
+	git_buf_dispose(&ref_name);
+	git_buf_dispose(&msg);
 }
 
 void test_online_push__initialize(void)
@@ -372,7 +376,7 @@ void test_online_push__initialize(void)
 
 	record_callbacks_data_clear(&_record_cbs_data);
 
-	cl_git_pass(git_remote_connect(_remote, GIT_DIRECTION_PUSH, &_record_cbs, NULL));
+	cl_git_pass(git_remote_connect(_remote, GIT_DIRECTION_PUSH, &_record_cbs, NULL, NULL));
 
 	/* Clean up previously pushed branches.  Fails if receive.denyDeletes is
 	 * set on the remote.  Also, on Git 1.7.0 and newer, you must run
@@ -393,7 +397,7 @@ void test_online_push__initialize(void)
 	}
 
 	git_remote_disconnect(_remote);
-	git_vector_free(&delete_specs);
+	git_vector_free_deep(&delete_specs);
 
 	/* Now that we've deleted everything, fetch from the remote */
 	memcpy(&fetch_opts.callbacks, &_record_cbs, sizeof(git_remote_callbacks));
